function [yyComponents] = ComputeMVWoldComponentsFromMA(WoldCoeff,shocks,J)
% PURPOSE: Compute components from Wold decompositon
% 
% -----------------------------------------------------
% USAGE: [yyComponents] = ComputeMVWoldComponentsFromMA(WoldCoeff,shocks,J)
% where:
% WoldCoeff = column vector containing Wold coefficients ordered as [\psi_0, \psi_1, ..., \psi_k]'
%             WoldCoeff may be of different length from shocks   
% shocks    = column vector containing shocks            ordered as [\epsilon_{-\infty}, ..., \epsilon_{t}, \epsilon_{t-1}, ...]', i.e. the first element is the oldest and the last element is the more recent 
%             the first "p" (# of lags) elements are NaN
% J         = number of components (actually J+1 since we have also \pi).
%             It must be that 2^J<WoldCoeff
% -----------------------------------------------------
% RETURNS: 
% yyComponents = the j-th column is associated with the j-th component, ordered as the shocks.
%                the first (1:p+2^jj-1) elements are "zero" because of 
%                            "p" lags plus 
%                              "2^jj-1" data points to initialize Redundant Haar
% 
% sum(yyComponents,2) ~ yy (original series)

HH = size(WoldCoeff,3);
if 2^J > HH
   error('Error: Input J must be less than log_2(length(WoldCoeff))')
end

[T,nvars]= size(shocks);          % sample size    %
shocksHaar_TEMP = NaN(T,nvars*(J+1));
for vv = 1:nvars
    shocksHaar_TEMP(:,(vv-1)*(J+1)+1:(vv)*(J+1)) =        RedundantHaarIso(J,shocks(:,vv),1); 
end

for jj = 1:J+1
    select_variables = [jj:J+1:nvars*(J+1)];
    shocksHaar{jj} = shocksHaar_TEMP(:,select_variables);
end

psiHaar = NaN(nvars,nvars,HH,J+1);
for vv = 1:nvars
    for vvv = 1:nvars
        psiHaar(vv,vvv,:,:) = flipud(RedundantHaarIso(J,flipud(squeeze(WoldCoeff(vv,vvv,:))),1)); % Recall we want psi^{1}_0 = (psi_0-psi_1)/sqrt(2) and WoldCoeff=[psi_0, psi_1, ...]'
    end
end

nobs = size(shocks,1);               if J>floor(log2(nobs)); error('Error: Input J must by <= than log_2(nobs)'); end
yyComponents = zeros(nobs,nvars,J+1);
for jj=1:J+1
    if jj~=J+1
        psi_scale = psiHaar(:,:,1:2^jj    :end,jj); max_num_shocks = size(psi_scale,3);
         for tt=nobs:-1:min(max_num_shocks*2^jj,nobs)
             temp_ = NaN(nvars,max_num_shocks);
             for iii=1:max_num_shocks
                 temp_(:,iii) = psi_scale(:,:,iii)*(shocksHaar{jj}(tt-2^jj*(iii-1),:))';
             end
             yyComponents(tt,:,jj) = nansum(temp_,2);
         end
        %-----------------------------------------------------------------%
        for tt=min(max_num_shocks*2^jj,nobs)-1:-1:1
            shocksHaarExtended =  shocksHaar{jj}(tt:-2^jj:1,:);
            shocksHaarExtended = [shocksHaarExtended; NaN(max_num_shocks-size(shocksHaarExtended,1),nvars)];
            temp_ = NaN(nvars,max_num_shocks);
            for iii=1:max_num_shocks
                temp_(:,iii) = psi_scale(:,:,iii)*(shocksHaarExtended(iii,:))';
            end
            yyComponents(tt,:,jj) = nansum(temp_,2);
        end
        %-----------------------------------------------------------------%
    else % permanent component
        psi_scale = psiHaar(:,:,1:2^(jj-1):end,jj); max_num_shocks = size(psi_scale,3);
        for tt=nobs:-1:min(max_num_shocks*2^(jj-1),nobs)
            temp_ = NaN(nvars,max_num_shocks);
            for iii=1:max_num_shocks
                 temp_(:,iii) = psi_scale(:,:,iii)*(shocksHaar{jj}(tt -2^(jj-1)*(iii-1),:))';
            end
            yyComponents(tt,:,jj) = nansum(temp_,2);
        end
        %-----------------------------------------------------------------%
        for tt=min(max_num_shocks*2^(jj-1),nobs)-1:-1:1
            shocksHaarExtended =                                                        shocksHaar{jj}(tt:-2^(jj-1):1,:);
            shocksHaarExtended = [shocksHaarExtended; NaN(max_num_shocks-size(shocksHaarExtended,1),nvars)];
            temp_ = NaN(nvars,max_num_shocks);
            for iii=1:max_num_shocks
                 temp_(:,iii) = psi_scale(:,:,iii)*(shocksHaarExtended(iii,:))';
            end
            yyComponents(tt,:,jj) = nansum(temp_,2);
        end
        %-----------------------------------------------------------------%
    end
%     
% end
% 
end

